// By Sandra Sims SDS Digital 2024
// Digit 0 = 1, Digits 0-5 are colors RGB,RGB
// Active notes on CHANNEL 2 start at 72, end at 95 but some are doubles...plays:76,83,88,95, but all blacks are active
// Records:73,75,78,80,82,85,87,90,92,94
// Plays  :72,74,77,79,81,84,86,89,91,93
// Buttons: on each strobe, bits 0,2 are record, bits 1,3 are play. So if bit 1 set then play button
// EEPROM can recall lasts if shut down, but LEDS will flash Blue?
// Pitch knob will send pitchnotes +/- 1 oct range. Possibly follow scales (min/maj/turkish etc). when centered will flash all white.
// Scale selection by bottom two (Play/REC) together
// LEVEL Knob> halfway increases midrange 50%(cc21,24) and removes bass EQ 100%(cc20,23)

#include <math.h>
#include <EEPROM.h>
#include <SPI.h>


unsigned long millisLast = 0;
unsigned long butStates = 0;
byte ledCol[10];
byte led[10]; //
byte address[16];//hold last written so can be or'ed to
byte dimmer = 13; //15 max
byte dimTOT = 50; // 5 seconds bright, then dims
byte whiteTOT = 0; //flash selection white
byte midLedTOT = 0; //midi LED timeout
int butWasTOT = 0; 
byte pNoteSent = 0;

byte ms100 = 0; //for timers etc
byte ms10 = 0;
int minute = 0; //minute count,600*100ms  for shutdown
byte seconds = 0;
byte jitter = 5;
byte shutDown = 0;
byte shutDownMinutes = 15; //default to 15 minutes. onLED with flash at 2 minutes
byte strobe = 0x01;
byte strobeCnt = 0;
byte butDbc[20];
int butHeld[20];
int  recButHeld[10];
int recTime = 0;
int playTime =  0;
int playTOT[10];
byte butFlags[5];
byte recNote[] = {73, 75, 78, 80, 82, 85, 87, 90, 92, 94};
byte playNote[] = {72, 74, 77, 79, 81, 84, 86, 89, 91, 93};
byte mChan = 0x01; //channel 2
byte velocity = 127;
int velLast = 512;
unsigned int pbl = 0;

boolean text = false;
boolean butStart = false;

byte aButPressed = 0;
byte ledUD = 0;
byte pitchLast = 48;
byte pNoteOff = 0;
byte bass = 64;
byte mid = 64;
byte pnLast = 100;

byte eepSel = 0; //to select memories 8 to 17, which is EEP 16-34 (ints)
byte eepSave = 0;

int playButRecover = 0;
byte warble = 0; // for special scale 10, warble up to + 1 oct, - 1 oct
boolean wState = false;

byte settingScale = 0;
byte scaleSet = 0; //scale offsets, scn[]=led#
byte allScale[] = {36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 48, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60};
byte sc0[] = {0, 0, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 0, 0};
byte whiteScale[] = {36, 36, 38, 38, 40, 41, 41, 43, 43, 45, 45, 47, 48, 48, 48, 50, 50, 52, 52, 53, 53, 55, 55, 57, 59, 59, 60};
byte sc1[] = {7, 7, 6, 6, 5, 4, 4, 3, 3, 2, 2, 1, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7};
byte blackScale[] = {37, 37, 39, 39, 39, 42, 42, 44, 44, 46, 46, 46, 48, 48, 48, 49, 49, 51, 51, 51, 54, 54, 56, 56, 58, 58, 58};
byte sc2[] = {5, 5, 4, 4, 4, 3, 3, 2, 2, 1, 1, 1, 0, 0, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 5};
byte M7scale[] = {36, 36, 36, 36, 40, 40, 40, 43, 43, 43, 46, 46, 48, 48, 48, 52, 52, 52, 52, 55, 55, 55, 58, 58, 58, 60, 60};
byte sc3[] = {4, 4, 4, 4, 3, 3, 3, 2, 2, 2, 1, 1, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4};
byte m7scale[] = {36, 36, 36, 36, 39, 39, 39, 43, 43, 43, 46, 46, 48, 48, 48, 51, 51, 51, 51, 55, 55, 55, 58, 58, 58, 60, 60};
//sc4 re-use sc3[]
byte T1scale[] = {36, 36, 38, 39, 39, 41, 41, 43, 44, 44, 47, 47, 48, 48, 48, 50, 50, 51, 51, 53, 53, 55, 56, 56, 59, 59, 60};
byte sc5[] = {7, 7, 6, 5, 5, 4, 4, 3, 2, 2, 1, 1, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 5, 5, 6, 6, 7};
byte T2scale[] = {36, 37, 37, 39, 39, 41, 41, 41, 44, 44, 46, 46, 48, 48, 48, 49, 49, 51, 51, 53, 53, 53, 56, 56, 58, 58, 60};
byte sc6[] = {6, 5, 5, 4, 4, 3, 3, 3, 2, 2, 1, 1, 0, 0, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6};
byte sc7[] = {0, 0, 10, 9, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 9, 10, 0, 0};
int freq[] = {400, 377, 356, 336, 317, 300 , 282 , 267, 251, 237, 224, 212, 200, 189, 178, 168, 158, 149, 141, 133, 125, 119, 112, 106, 100};

void setup() {
  pinMode(12, INPUT_PULLUP);

  pinMode(9, OUTPUT);
  pinMode(10, OUTPUT); //keep alive


  pinMode(2, OUTPUT); //switch strobe
  pinMode(3, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  pinMode(6, OUTPUT);
  pinMode(A2, INPUT_PULLUP); //4 switches per strobe
  pinMode(A3, INPUT_PULLUP);
  pinMode(A4, INPUT_PULLUP);
  pinMode(A5, INPUT_PULLUP);
  delay(50);
  SPI.setClockDivider(SPI_CLOCK_DIV8);//4 4uS (changed) per byte should be ok. *Must have 20uS+space between Sends {ORIG=32 untested digPots}
  SPI.setDataMode(SPI_MODE3);//
  SPI.begin();  // initialize SPI:

  bitClear(PORTD, 6); analogWrite(10, 0);
  if ((PINC & 0x3C) != 0x3C) {
    analogWrite(10, 128); // .only turn on Battery if was by buttons! zero once powering down
    butStart = true;
  }

//debug
  text = false;//true;//

  if (!text)Serial.begin(31250);
  if (text)Serial.begin(115200);
  ledSetup();



  PORTD |= 0x7C; PORTD &= 0x04;;
  //for(byte i=1;i<5;i++){
  //  address[i]=0x60;
  //}

  onLed(1);
  if (!text) {
    midiCC(9, 0); //set "BLEND" fully right
    delay(5);
    for (byte i = 21; i < 27; i++) {
      midiCC(i, 64); //set all EQ's to center. pot position will update
      delay(5);
    }
    midiCC(9, 0); //set "BLEND" fully right again to be sure ;)
  }
  //for(byte i =1;i<128;i++){ //clear eep, as had data for some reason
  //  EEPROM.write(i,255);
  //}
  if(EEPROM.read(0)<10)scaleSet = EEPROM.read(0);
  if (text) {
    for (byte i = 0x10; i < 50; i += 2) {
      Serial.print((i - 16) / 2); Serial.print("\t"); Serial.println(EEPROM.read(i) * 256 + EEPROM.read(i + 1));
    }
  }
}

void loop() {
  if (millis() != millisLast) {
    millisLast = millis();

    if (recTime)recTime++;
    if (playTime)playTime++;
    if (butWasTOT > 0)butWasTOT--;

    warble++; if (warble == 57 && scaleSet == 9 && butWasTOT && pNoteSent != 48) { //57 is appx 130bpm
      warble = 0; pbl += 10; //to trigger a send of pitch note
      if (wState) {
        wState = false;
      } else {
        wState = true;
      }
    }

    //===============================================ms100===================================
    ms100++; if (ms100 > 99) {
      ms100 = 0;

      seconds++; if (seconds > 9) {
        seconds = 0;
        if (jitter < 12)jitter++;

      }

      minute++; if (minute > 599) {
        minute = 0; shutDown++; if (shutDown == shutDownMinutes && butStart)shutDownNow(); //only if was powered up with button
        ledSetup(); //in case led chip fails, as it seems to a lot
      }
      eepSave++; if (eepSave > 99) { //update one every 10 seconds
        int e = EEPROM.read(0x10 + (eepSel * 2)); e *= 256; e += EEPROM.read(0x11 + (eepSel * 2));
        if (recButHeld[eepSel] != 0 && e != recButHeld[eepSel]) {
          EEPROM.write(0x10 + (eepSel * 2), recButHeld[eepSel] / 256); EEPROM.write(0x11 + (eepSel * 2), recButHeld[eepSel]);
          if (text) {
            Serial.print(F("Writing:")); Serial.print(0x10 + (eepSel * 2), HEX); Serial.print(F(" with")); Serial.println(recButHeld[eepSel]);
          }
        }
        eepSave = 0; eepSel++; if (eepSel > 9)eepSel = 0;
      }

      //
      if (shutDownMinutes - shutDown < 3 && butStart) {
        if (!(address[5] & 0x20)) {
          onLed(1);
        } else {
          onLed(0);
        }
      }
      if (dimTOT) {
        if (dimTOT == 50) {
          dimmer = 13;
          bitClear(PORTB, 1);
          SPI.transfer(0x0a);  //intensity
          SPI.transfer(dimmer); address[0x0a] = dimmer;
          bitSet(PORTB, 1);
          shutDown = 0; //reset shutdown
        }
        dimTOT--; if (dimTOT < 10) {
          dimmer--; if (dimmer < 3)dimmer = 3;
          bitClear(PORTB, 1);
          SPI.transfer(0x0a);  //intensity
          SPI.transfer(dimmer); address[0x0a] = dimmer;
          bitSet(PORTB, 1);
        }
      }


    }
    //======================= end ms 100 ========================




    // ========================================== ms10 ===========================================
    ms10++; if (ms10 > 9) {
      ms10 = 0;
      //                             ================scales=======================

      int p0 = analogRead(A1); unsigned int pb = p0;  p0 /= 38; //27 possibilities 12-3(center)-12
      //if (pitchLast != p0 || (scaleSet > 6 && pb > 5 && (pb > pbl + 5 || pb < pbl - 5))) {
      if (pb > jitter && (pb > pbl + jitter || pb < pbl - jitter)) {
        if (settingScale && p0 < 1)p0 = 1; // no zero setting
        pbl = pb + 2; if (pb > pbl + jitter)pbl -= 4;
        if (jitter > 5)jitter--;
        byte pNote = allScale[p0]; byte pn = sc0[p0];//pNote - 36; //so shows relative notes
        int war = pNote;

        switch (scaleSet) {
          case 1: pNote = whiteScale[p0]; pn = sc1[p0]; break;
          case 2: pNote = blackScale[p0]; pn = sc2[p0]; break;
          case 3: pNote = M7scale[p0]; pn = sc3[p0]; break;
          case 4: pNote = m7scale[p0]; pn = sc3[p0]; break;//same led
          case 5: pNote = T1scale[p0]; pn = sc5[p0]; break;
          case 6: pNote = T2scale[p0]; pn = sc6[p0]; break;
          case 7: pNote = 48; pn = sc7[p0]; break; // PitchWheel +/- 2
          case 8: pNote = 48; pn = sc7[p0]; break; // PitchWheel +/- 12
          case 9: war = war - 48;
            if (wState) {
              pNote = 48 - war;
            } else {
              pNote = 48 + war;
            }
            break;
        }

        while (whiteTOT) whiteOff();

        if (pn > 0 )ledUpdate(pn - 1, 7); pnLast = pn;//&& pn != pnLast << removed as skips ends...
        whiteTOT = 30; if (pNote == 36 || pNote == 60)whiteTOT = 90;
        if (settingScale)whiteTOT = 100;
        dimTOT = 50;
        //}
        if (scaleSet < 7 || scaleSet == 9) {
          if (text && !settingScale) {
            Serial.print(F("Pitch Note On")); Serial.println(pNote);
          }
          if (!text && !settingScale) {
            midiNoteOn(pNote); pNoteOff = pNote; pNoteSent = pNote;
          }

        }
        else if (!settingScale) { //pitchbending!

          if (pb > 470 && pb < 554)pb = 512; //snap to center! 1/12 a full note
          pb = pb << 4;
          if (scaleSet == 7) { //+/-2
            int pb2 = pb - 8192; pb2 /= 3; pb = pb2 + 8192;
          }
          byte lsb = pb & 0x7f; pb = pb >> 7; byte msb = pb & 0x7f;
          if (text) {
            Serial.print(F("PW:")); Serial.print(msb); Serial.print(" "); Serial.println(lsb);
          } else { //send channel 2 pitchbend message
            midiLed(1); Serial.write(0xE0 + mChan); Serial.write(lsb); Serial.write(msb);
          }
        }
        pitchLast = p0;
      }
      if (whiteTOT) {
        whiteOff();
      }
      if (midLedTOT) {
        midLedTOT--; if (midLedTOT == 0)midiLed(0);
      }
    } //end 10ms


    if (playTOT[ms10] != 0) {
      playTOT[ms10] -= 10; if (playTOT[ms10] <= 0) {
        playTOT[ms10] = 0; if (!(ledCol[ms10] & 0x80)) { //only of was released, if held then ignore
          ledCol[ms10] &= 0x7D; //kill flag too
          if (recButHeld[ms10])ledCol[ms10] |= 0x04; //return to blue if was active
          ledUpdate(ms10,  ledCol[ms10]);
        }
      }

    }

    //================ buttons =====================================================================================

    if ((ms100 & 0x01))button(); //every 2mS for now

    ledUD++; if (ledUD > 9)ledUD = 0;
    unsigned long b = butStates >> (ledUD * 2);
    //rec++
    if ((b & 0x01) && !(ledCol[ledUD] & 0x01)) { //=======recleds
      //Serial.println(b,BIN);
      if (!(butStates & 0x80000)) { //as long as play button not held
        //record start!
        ledCol[ledUD] |= 0x01; ledCol[ledUD] &= 0xFB; ledUpdate(ledUD, ledCol[ledUD]);
        recButHeld[ledUD] = 1; recTime = 1;
        if (text) {
          Serial.print(F("Rec#")); Serial.println(ledUD); Serial.print(F(" b=")); Serial.println(butStates);
        } else {
          midiNoteOn(recNote[ledUD]);
        }
        butStates = 0;
      }
      else if (ledUD == 9) {
        scaleSet = 0; settingScale = 1;
        for (byte i = 0; i < 10; i++) {
          ledUpdate(i, 0);
        }
      }
    }
    else if (!(b & 0x01) && (ledCol[ledUD] & 0x01)  && !settingScale) { //temp test!
      ledCol[ledUD] &= 0xFE; ledCol[ledUD] |= 0x04; ledUpdate(ledUD, ledCol[ledUD]); //will always be blue after a record
      recButHeld[ledUD] = recTime; recTime = 0;
      if (recButHeld[ledUD] < 200) { //erase!
        recButHeld[ledUD] = 0; ledCol[ledUD] &= 0xFB; //turn off blue. No longer playable.
        ledUpdate(ledUD, ledCol[ledUD]);
      }
      if (text) {
        Serial.print(F("Rec Time:")); Serial.println(recButHeld[ledUD]);
      } else {
        midiNoteOff(recNote[ledUD]);
      }
    }

    //play++
    b = butStates >> ((ledUD * 2) + 1);
    if ((b & 0x01) && !(ledCol[ledUD] & 0x80)) { //=======playleds
      if (recButHeld[ledUD]) { //only play if was recorded!
        ledCol[ledUD] |= 0x02; ledCol[ledUD] &= 0xFB; ledUpdate(ledUD,  ledCol[ledUD]); //kill blue
        playTime = 1;
        unsigned long len = freq[pNoteSent - 36]; len *= recButHeld[ledUD]; len /= 200UL; playTOT[ledUD] = len;
        if (text) {
          Serial.print(F("play.")); Serial.println(playNote[ledUD]);
        } else {
          midiNoteOn(playNote[ledUD]);
        }
        ledCol[ledUD] |= 0x80;//flag
        butWasTOT = recButHeld[ledUD] * 2; // will stop sending warble after a bit
        warble = 0; wState = false; //next is going up
      } else {
        //Serial.print(ledCol[ledUD]); Serial.println(playButRecover);
        playButRecover++; if (ledCol[ledUD] == 0 && playButRecover > 1000) { //every 10ms makes 10 seconds hold to unlock
          int e = EEPROM.read(0x10 + (ledUD * 2)); e *= 256; e += EEPROM.read(0x11 + (ledUD * 2));
          recButHeld[ledUD] = e + 2; //auto-saved size
          ledCol[ledUD] |= 0x04; ledUpdate(ledUD,  ledCol[ledUD]); //turn on blue
        }
      }
    }
    else if (!(b & 0x01) && (ledCol[ledUD] & 0x80)) { //flag instead of green 0x02
      ledCol[ledUD] &= 0x7F;
      if (playTime > 299 || playTime > recButHeld[ledUD]) { //if tapped then will time out led
        ledCol[ledUD] &= 0xFD;
        if (recButHeld[ledUD])ledCol[ledUD] |= 0x04; //return to blue if was active
        ledUpdate(ledUD,  ledCol[ledUD]);
        playTOT[ledUD] = 0; if (text)Serial.print("held..");
      }
      playTime = 0; //send midi
      if (text) {
        Serial.print(F("play off:")); Serial.println(playNote[ledUD]);
      } else {
        midiNoteOff(playNote[ledUD]);
      }
      playButRecover = 0;
    }

    if (settingScale == 2 && butStates) { // in setting scale mode. any button ends it
      scaleSet = pnLast - 1; settingScale = 0; EEPROM.write(0, scaleSet);
      for (byte i = 0; i < 10; i++) { //put back blues here!
        ledUpdate(i, ledCol[i]);
      }
      if (scaleSet > 6) {
        midiNoteOn(48); midiNoteOff(48); //center pitch notes
      } else { //center pitch bend!
        midiLed(1); Serial.write(0xE0 + mChan); Serial.write(0); Serial.write(0x40);
      }
    }
    if (settingScale == 1 && !butStates)settingScale = 2;

    //}
    if (millis() < 2000 && (ms100 & 0x02)) {
      byte ledSel = random(10); byte ledColor = random(8);
      //Serial.print("LED#"); Serial.print(ledSel); Serial.print(" Color:"); Serial.print(ledColor); Serial.print("\t > ");
      ledUpdate(ledSel, ledColor);
    }
    else if (millis() > 2000 && millis() < 2200) {
      for (byte i = 0; i < 10; i++) {
        ledUpdate(i, 0);
      }
    }



    int a0 = analogRead(A0);
    if (a0 - velLast > 10 || a0 - velLast < -10) {
      velLast = a0;
      a0 /= 4;
      velocity = a0;
      if (a0 > 127) {
        a0 -= 128;
        velocity = 127;
        bass = (128 - a0) / 2; //starts at 64 and drops to 0 (-15db)
        mid = 64 + a0 / 4;
      }
      if (!text) {
        midiLed(1); midiCC(20, bass); midiCC(23, bass); midiCC(21, mid); midiCC(24, mid); //midi cc outs
      } else {
        Serial.print("Vel:"); Serial.print(velocity); Serial.print(" mid:"); Serial.print(mid); Serial.print(" bass:"); Serial.println(bass);
      }

    }


  }//end 1mS

}

void ledUpdate(byte ledNum, byte color) { //color 0=off, 0x01=red,0x02=green,0x04=blue,0x03=yellow,0x05=violet,0x06=aqua,0x07=white
  byte dig = 1; byte segs = ledNum; color &= 0x0f;
  if (ledNum > 4) {
    dig = 4; segs = ledNum - 5;
  }
  byte segsBit = (0x01 << segs);
  byte bits = color;
  for (byte i = 0; i < 3; i++) {//RGB
    address[dig] &= (~segsBit); if ((bits & 0x01))address[dig] |= segsBit;
    bitClear(PORTB, 1);
    SPI.transfer(dig);  //in case in display test mode
    SPI.transfer(address[dig]);
    bitSet(PORTB, 1); delayMicroseconds(30);
    //if (i == 0)Serial.print(" [R]");  if (i == 1)Serial.print(" [G]");  if (i == 2)Serial.print(" [B]");
    //Serial.print(dig); Serial.print(":"); Serial.print(address[dig], HEX);
    bits = bits >> 1; dig++;

  }
  //ledNum=0,red. segs=1, bits=1. { ad[1]&=FE,ad[1]|=1. SPI(1,1G)...ad[2]&=FE,ad[2]same. SPI(2,0G)...}
  //ledNum=1,green, segs=2,bits=2 { ad[1]&=FD,ad[1]same.SPI(1,0F)...ad[2]&=FD,ad[2]|=2. SPI(2,1F)...}
  //ledNum=9,blue, seg=9,4,0x10. bits=4 {ad[4]&=EF,ad[4]same.SPI(4,0C)..SPI(5,0C)..SPI(6,1C) ..}
  //Serial.println();
}


void whiteOff() {
  whiteTOT--; if (whiteTOT == 0) {
    for (byte i = 0; i < 10; i++) { //put back blues here!
      ledUpdate(i, ledCol[i]);
    }
    if (text && pNoteOff && !settingScale) {
      Serial.print(F("Pitch Note Off: ")); Serial.println(pNoteOff);
    }
    if (!text && pNoteOff && !settingScale) {
      midiNoteOff(pNoteOff); pNoteOff = 0;
    }

  }
}


void midiLed(byte s) { //somehow on digit 5!
  address[5] &= 0xBF; if (s != 0)address[5] |= 0x40;
  bitClear(PORTB, 1);
  SPI.transfer(5);  //in case in display test mode
  SPI.transfer(address[5]);
  bitSet(PORTB, 1); delayMicroseconds(30);
  midLedTOT = 3; //10mS
}
void onLed(byte s) {
  address[5] &= 0xDF; if (s != 0)address[5] |= 0x20;
  bitClear(PORTB, 1);
  SPI.transfer(5);  //in case in display test mode
  SPI.transfer(address[5]);
  bitSet(PORTB, 1);
}


void button() { //which button(s) on present strobe, b active=low... b flags are 0x01,0x04 for REC, 0x02,0x08 for play
  // set flag in unsigned long for buttons
  byte buts = (PINC & 0x3C);
  //byte buts = 0x3c;
  if ((buts & 0x3c) != 0x3c)dimTOT = 50; // 5 seconds bright, then dims

  //Serial.println(128 + buts, BIN);
  byte fix = (buts & 0x0f); fix |= 0x30;
  if (!(buts & 0x10))fix &= 0xDF; //reverse A4/A5 pinout fuckup.
  if (!(buts & 0x20))fix &= 0xEF;
  fix = fix >> 2; fix = (~fix);

  //unsigned long sBit = 0x01; sBit = sBit << (strobeCnt * 4); // --->proble is others are getting reset
  // Serial.println(128 + fix, BIN);
  byte sc4 = strobeCnt * 4;
  unsigned long blank = 0x0000000F; blank = blank << sc4; butStates &= (~blank);
  blank = (fix & 0x0f); blank = blank << sc4; butStates |= blank;
  //Serial.println(0x10000000 + butStates, BIN);

  strobe = strobe << 1; if (!(strobe & 0x1F))strobe = 0x01; byte str = strobe << 2;
  //Serial.println(str,BIN);
  DDRD &= 0x83; DDRD |= str;
  str = ~str; PORTD |= 0x7C; PORTD &= str;//
  //Serial.println(str,HEX);
  strobeCnt++; if (strobe == 0x01)strobeCnt = 0;
  //delay(10);
}

void midiNoteOn(byte n) {
  midiLed(1); if (text)return;
  if (velocity == 0) velocity = 1;
  Serial.write(0x90 + mChan); Serial.write(n); Serial.write(velocity);
}
void midiNoteOff(byte n) {
  midiLed(1); if (text)return;
  Serial.write(0x80 + mChan); Serial.write(n); Serial.write(64);
}
void midiCC(byte c, byte val) {
  midiLed(1); if (text)return;
  Serial.write(0xB0 + mChan); Serial.write(c); Serial.write(val);
}

void ledSetup() { //set level, scan limit, etc. Maybe try a resistor off "digit 4?" to led circle to make extra bright?
  bitClear(PORTB, 1);
  SPI.transfer(0x0f);  //in case in display test mode
  SPI.transfer(0x00); address[0x0f] = 0x00;
  bitSet(PORTB, 1);
  delayMicroseconds(30); //added aug31
  bitClear(PORTB, 1);
  SPI.transfer(0x0c);  //shutdown=run normally
  SPI.transfer(0x01); address[0x0c] = 0x01;
  bitSet(PORTB, 1);
  delayMicroseconds(30);
  bitClear(PORTB, 1);
  SPI.transfer(0x0b);  //scan limit=6
  SPI.transfer(0x05); address[0x0b] = 0x05;
  bitSet(PORTB, 1);
  delayMicroseconds(30);
  bitClear(PORTB, 1);
  SPI.transfer(0x09);  //decode mode
  SPI.transfer(0x00); address[0x09] = 0x00;
  bitSet(PORTB, 1);
  delayMicroseconds(30);
  bitClear(PORTB, 1);
  SPI.transfer(0x0a);  //intensity
  SPI.transfer(dimmer); address[0x0a] = dimmer;
  bitSet(PORTB, 1);
}
void shutDownNow() {
  //do any state save to EEPROM? These will be recalled if play button is pressed with no record before it...
  analogWrite(10, 0); //will shutdown.
}
